// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Batch.Protocol.Models
{
    using System.Linq;

    /// <summary>
    /// A reference to an Azure Virtual Machines Marketplace image.
    /// </summary>
    public partial class ImageReference
    {
        /// <summary>
        /// Initializes a new instance of the ImageReference class.
        /// </summary>
        public ImageReference() { }

        /// <summary>
        /// Initializes a new instance of the ImageReference class.
        /// </summary>
        /// <param name="publisher">The publisher of the Azure Virtual
        /// Machines Marketplace image.</param>
        /// <param name="offer">The offer type of the Azure Virtual Machines
        /// Marketplace image.</param>
        /// <param name="sku">The SKU of the Azure Virtual Machines
        /// Marketplace image.</param>
        /// <param name="version">The version of the Azure Virtual Machines
        /// Marketplace image.</param>
        public ImageReference(string publisher, string offer, string sku, string version = default(string))
        {
            Publisher = publisher;
            Offer = offer;
            Sku = sku;
            Version = version;
        }

        /// <summary>
        /// Gets or sets the publisher of the Azure Virtual Machines
        /// Marketplace image.
        /// </summary>
        /// <remarks>
        /// For example, Canonical or MicrosoftWindowsServer.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "publisher")]
        public string Publisher { get; set; }

        /// <summary>
        /// Gets or sets the offer type of the Azure Virtual Machines
        /// Marketplace image.
        /// </summary>
        /// <remarks>
        /// For example, UbuntuServer or WindowsServer.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "offer")]
        public string Offer { get; set; }

        /// <summary>
        /// Gets or sets the SKU of the Azure Virtual Machines Marketplace
        /// image.
        /// </summary>
        /// <remarks>
        /// For example, 14.04.0-LTS or 2012-R2-Datacenter.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "sku")]
        public string Sku { get; set; }

        /// <summary>
        /// Gets or sets the version of the Azure Virtual Machines Marketplace
        /// image.
        /// </summary>
        /// <remarks>
        /// A value of 'latest' can be specified to select the latest version
        /// of an image. If omitted, the default is 'latest'.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "version")]
        public string Version { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (Publisher == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Publisher");
            }
            if (Offer == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Offer");
            }
            if (Sku == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Sku");
            }
        }
    }
}
