// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator 0.17.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.Insights.Models
{
    using System.Linq;

    /// <summary>
    /// Autoscale profile.
    /// </summary>
    public partial class AutoscaleProfile
    {
        /// <summary>
        /// Initializes a new instance of the AutoscaleProfile class.
        /// </summary>
        public AutoscaleProfile() { }

        /// <summary>
        /// Initializes a new instance of the AutoscaleProfile class.
        /// </summary>
        /// <param name="name">the name of the profile.</param>
        /// <param name="capacity">the number of instances that can be used
        /// during this profile.</param>
        /// <param name="rules">the collection of rules that provide the
        /// triggers and parameters for the scaling action. A maximum of 10
        /// rules can be specified.</param>
        /// <param name="fixedDate">the specific date-time for the profile.
        /// This element is not used if the Recurrence element is
        /// used.</param>
        /// <param name="recurrence">the repeating times at which this profile
        /// begins. This element is not used if the FixedDate element is
        /// used.</param>
        public AutoscaleProfile(string name, ScaleCapacity capacity, System.Collections.Generic.IList<ScaleRule> rules, TimeWindow fixedDate = default(TimeWindow), Recurrence recurrence = default(Recurrence))
        {
            Name = name;
            Capacity = capacity;
            Rules = rules;
            FixedDate = fixedDate;
            Recurrence = recurrence;
        }

        /// <summary>
        /// Gets or sets the name of the profile.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the number of instances that can be used during this
        /// profile.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "capacity")]
        public ScaleCapacity Capacity { get; set; }

        /// <summary>
        /// Gets or sets the collection of rules that provide the triggers and
        /// parameters for the scaling action. A maximum of 10 rules can be
        /// specified.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "rules")]
        public System.Collections.Generic.IList<ScaleRule> Rules { get; set; }

        /// <summary>
        /// Gets or sets the specific date-time for the profile. This element
        /// is not used if the Recurrence element is used.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "fixedDate")]
        public TimeWindow FixedDate { get; set; }

        /// <summary>
        /// Gets or sets the repeating times at which this profile begins.
        /// This element is not used if the FixedDate element is used.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "recurrence")]
        public Recurrence Recurrence { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (Name == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Name");
            }
            if (Capacity == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Capacity");
            }
            if (Rules == null)
            {
                throw new Microsoft.Rest.ValidationException(Microsoft.Rest.ValidationRules.CannotBeNull, "Rules");
            }
            if (this.Capacity != null)
            {
                this.Capacity.Validate();
            }
            if (this.Rules != null)
            {
                foreach (var element in this.Rules)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
            if (this.FixedDate != null)
            {
                this.FixedDate.Validate();
            }
        }
    }
}
