// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Search.Models
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using Newtonsoft.Json;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Microsoft.Rest.Azure;

    /// <summary>
    /// Defines parameters for an Azure Search index that influence scoring in
    /// search queries.
    /// <see href="https://msdn.microsoft.com/library/azure/dn798928.aspx" />
    /// </summary>
    public partial class ScoringProfile
    {
        /// <summary>
        /// Initializes a new instance of the ScoringProfile class.
        /// </summary>
        public ScoringProfile() { }

        /// <summary>
        /// Initializes a new instance of the ScoringProfile class.
        /// </summary>
        public ScoringProfile(string name, TextWeights textWeights = default(TextWeights), IList<ScoringFunction> functions = default(IList<ScoringFunction>), ScoringFunctionAggregation? functionAggregation = default(ScoringFunctionAggregation?))
        {
            Name = name;
            TextWeights = textWeights;
            Functions = functions;
            FunctionAggregation = functionAggregation;
        }

        /// <summary>
        /// Gets or sets the name of the scoring profile.
        /// </summary>
        [JsonProperty(PropertyName = "name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets parameters that boost scoring based on text matches
        /// in certain index fields.
        /// </summary>
        [JsonProperty(PropertyName = "text")]
        public TextWeights TextWeights { get; set; }

        /// <summary>
        /// Gets or sets the collection of functions that influence the
        /// scoring of documents.
        /// </summary>
        [JsonProperty(PropertyName = "functions")]
        public IList<ScoringFunction> Functions { get; set; }

        /// <summary>
        /// Gets or sets a value indicating how the results of individual
        /// scoring functions should be combined. Defaults to "Sum". Ignored
        /// if there are no scoring functions. Possible values include:
        /// 'sum', 'average', 'minimum', 'maximum', 'firstMatching'
        /// </summary>
        [JsonProperty(PropertyName = "functionAggregation")]
        public ScoringFunctionAggregation? FunctionAggregation { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (Name == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Name");
            }
            if (this.TextWeights != null)
            {
                this.TextWeights.Validate();
            }
            if (this.Functions != null)
            {
                foreach (var element in this.Functions)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
        }
    }
}
